function H = uu2homog(u1,u2,s)

%UU2HOMOG  Estimates a homography between two planar 2D point sets.
%          H = UU2HOMOG(u1,u2) computes the nearest H in LS sense so that
%            u2 = p2e( H * e2p(u1) ),   (see P2E, E2P)
%          where
%            H ... Size (3,3). Homography matrix.
%            u1, u2 ... Sizes (2,N). Planar point sets in cartesian coords.
%
%          The values u1, u2 must be near 1 for the LS estimator to work properly.
%          H = UU2HOMOG(u1,u2,'norm') performs also the normalizations that ensures it,
%          or you can use NORMU outside UU2HOMOG.
%
%          See also NORMU.

%       Author       : (c) T. Werner and T. Pajdla, {werner,pajdla}@cmp.felk.cvut.cz
%                      07/09/98
%                      Center for Machine Perception, Czech Technical University, Prague
%       Documentation: ftp://cmp.felk.cvut.cz/pub/cmp/articles/werner/tr97-138.ps.gz     
%       Language     : Matlab 5.1, (c) MathWorks                         
%       Last change  : $Id: uu2homog.m,v 1.1 1998/09/07 15:54:15 Matlab Exp $
%       Status       : Ready ($Source: /home.others/Matlab/Matlab/radial/RCS/uu2homog.m,v $)


if size(u1)~=size(u2), error('Bad sizes of input parameters.'); end
N = size(u2,2);


%%
 % Normalize u1, u2 so that they are near 1.
 % See NORMU for details.
 %%

if nargin<3, s = ''; end
if strcmp(s,'norm')
  B1 = normu(u1);  u1 = p2e(B1*e2p(u1));
  B2 = normu(u2);  u2 = p2e(B2*e2p(u2));
else
  B1 = eye(3);
  B2 = B1;
end  


%%
 % (In the following Eqns we assume that size(u1)=size(u2)=[2 1].)
 % The homographic transform can be written as
 %
 %   [u2; 1] = a*H*[u1; 1]
 %
 % After eliminating the scale factor a it can be rewritten as
 %
 %   u2(1) = H(1,:)*[u1; 1] / H(3,:)*[u1; 1],
 %   u2(2) = H(2,:)*[u1; 1] / H(3,:)*[u1; 1],
 %
 % it can be further written as
 %
 %   A*reshape(H',9,1) = 0,
 %
 % where
 %
 %   A = [ u1(1) u1(2) 1    0     0   0  -u2(1)*[u1(1) u1(2) 1]
 %           0     0   0  u1(1) u1(2) 1  -u2(2)*[u1(1) u1(2) 1] ]
 %
 % This homogeneous system of linear eqns is overdetermined for number of 
 % point pairs > 4.
 %
 % For the least squares to work properly, the coords of u1, u2 must
 % be near 1 ===> norming u1, u2 needed.
 %%

% form matrix A
U1 = [u1' ones(N,1)];
U0 = zeros(size(U1));
A = [ U1 U0 -U1.*(u2(1,:)'*[1 1 1])
      U0 U1 -U1.*(u2(2,:)'*[1 1 1]) ];


%%
 % Solving the eqns system by SVD or EIG.
 % SVD is more stable but prohibitively slow for large number of points.
 %%
if 0
  % SVD solution
  [u,s,v] = svd(A);
  Hn_ = v(:,9);
else
  % EIG solution
  [v,s] = eig(A'*A);
  [void,i] = sort(abs(diag(s)));
  Hn_ = v(:,i(1));
end

H = inv(B2) * reshape(Hn_,3,3)' * B1;


return
